/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   APACHE
* @PACKAGE  MONMGMT 
*
* @FILE     monitor_exe.c

* @ABSTRACT
* Apache MONitoring ManaGeMenT (MONMGMT) : C program launched by the handler
*
* This file contains:
* the C program that will be launched by the Apache handler
*
* @INCLUDE


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <grp.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdarg.h>
#include <errno.h>

#include "gcn_uptyp.h"
#include "gcn_upcon.h"
#include "osx_upext.h"
#include "cdx_upcon.h"
#include "cdx_uptyp.h"
#include "cdx_upext.h"
#include "com_upext.h"

#include "monitor_prcon.h"
#include "monitor_prmes.h"
#include "monitor_upext.h"

extern CSICHAR *szAction;
extern CSICHAR *szSiteString;
extern CSICHAR *szUserSite;
extern CSICHAR *szGtwString;
extern CSICHAR *szRapString;
extern CSICHAR *szProfileString;

extern CSICHAR48 szWebDomain;
extern CSIBOOLEAN OSInit;
extern CSIBOOLEAN CDXBind;


static CSIVOID HandleError(CSILONG lErrorId, ...)
{
    va_list arguments;
    CSICHAR1024 szMsg;
    CSILONG lReturnCode;
    CSILONG lReasonCode;

    va_start(arguments, lErrorId);
    vsprintf(szMsg
            ,MONMGMT_ERROR_MSGS[lErrorId - 1]
            ,arguments
            );
    va_end(arguments);

    PrintAdminServiceBegin(CSIFAILED);
    PrintErrorMessage(lErrorId, szMsg);
    PrintAdminServiceEnd();

    if (CSITRUE == CDXBind)
    {
        CDX_unbind(NULL, &lReturnCode, &lReasonCode);
        CDXBind = CSIFALSE;
    }

    if (CSITRUE == OSInit)
    {
        OS_term(&lReturnCode, &lReasonCode);
        OSInit = CSIFALSE;
    }

    exit(1);
}


CSIVOID PrintHostMonitor(CSICHAR *szHostName, CSICHAR *szTimestamp, CSIBOOLEAN multiModeBackup)
{
    int fd[2];
    pid_t pid;


    if (pipe(fd) < 0)
        HandleError(MON_ERROR_INTERNAL, "pipe", errno);
     
    if ((pid = fork())  < 0)
        HandleError(MON_ERROR_INTERNAL, "fork", errno);
    else if (pid == 0)
    {
        /* child */
        CSICHAR256 szRRTask;
        CSICHAR256 szASYNCTask;
        CSICHAR256 szHTTPTask;
        CSICHAR256 szHTTPSTask;
        CSICHAR256 szAPPTask;
        CSICHAR256 szUSRTask;
        CSICHAR256 szQUETask;
        CSICHAR256 szNS50Task;

        close(fd[0]);
        if (STDOUT_FILENO != fd[1])
        {   
            if (STDOUT_FILENO != dup2(fd[1], STDOUT_FILENO))
                HandleError(MON_ERROR_INTERNAL, "dup2", errno);
            close(fd[1]);
        }   

        sprintf(szRRTask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_RR
               );
        sprintf(szASYNCTask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_ASYNC
               );
        sprintf(szHTTPTask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_HTTP
               );
        sprintf(szHTTPSTask
               ,"%s/%s.%s"    
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_HTTPS
               );           
        sprintf(szAPPTask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_APP
               );
        sprintf(szUSRTask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_USR
               );
        sprintf(szQUETask
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName
               ,MON_TASK_QUE
               );
        sprintf(szNS50Task  
               ,"%s/%s.%s"
               ,getenv("CCN_MONITOR_BBDIR")
               ,szHostName                
               ,MON_TASK_NS50
               );
        if (multiModeBackup == CSITRUE)
        {
           execlp(MON_PARSER
                 ,MON_PARSER
                 ,szAPPTask
                 ,szQUETask
                 ,szNS50Task
                 ,(char *) 0);
        }
        else
        {
          execlp(MON_PARSER
                ,MON_PARSER
                ,szRRTask
                ,szASYNCTask
                ,szHTTPTask
                ,szHTTPSTask
                ,szAPPTask
                ,szUSRTask
                ,szQUETask
                ,szNS50Task
                ,(char *) 0);
        }
        HandleError(MON_ERROR_INTERNAL, "execlp", errno);
    }
    else
    {  
        /* parent */
        FILE *fpipe   = NULL;
        char line[MAXLINE];
        int iStatus;
        
        close(fd[1]);

        if ((fpipe = fdopen(fd[0], "r")) == NULL)
            HandleError(MON_ERROR_INTERNAL, "fdopen", errno);
        
        PrintAdminServiceBegin(CSISUCCESS);
       
        while(fgets(line, MAXLINE, fpipe) != NULL)
            printf("%s", line);
        fflush(stdout);
        
        PrintAdminServiceEnd();

        fclose(fpipe);
        close(fd[0]);

        if (waitpid(pid, &iStatus , WUNTRACED) != pid)
            HandleError(MON_ERROR_INTERNAL, "waitpid", errno);
        if (!WIFEXITED(iStatus))
            HandleError(MON_ERROR_INTERNAL, "waitpid", iStatus);
        if (0 != WEXITSTATUS(iStatus))
            HandleError(MON_ERROR_INTERNAL, "waitpid", iStatus);

    }   

    return;
}

