/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   CCN
* @PACKAGE  LI (Local Intelligence engine)
*
* @FILE     li_prext.h

*
* @ABSTRACT
* Local Intelligence PRivate EXTernal declarations.
* Contains the prototypes of the inner routines of the LI engine, such as:
* - <@LOCALFUN li_OperationDispatcher>;
* - <@LOCALFUN li_AllocMsgContext>;
* - <@LOCALFUN li_FreeMsgContext>;
* - <@LOCALFUN li_MsgRoutScheduler>;
* - <@LOCALFUN li_LoopDetection>;
* - <@LOCALFUN li_ACMScheduler>;
* - <@LOCALFUN li_GetConvInfo>;
* - <@LOCALFUN li_MsgConvertToPivot>;
* - <@LOCALFUN li_MsgConvertFromPivot>;
* - <@LOCALFUN li_GetMsgInfoForVAS>;
* - <@LOCALFUN li_MsgConvert>;
* - <@LOCALFUN li_SaveOriginalData>;
* - <@LOCALFUN li_RestoreOriginalData>;
* - <@LOCALFUN li_BuildFileName>;
* - <@LOCALFUN li_IsQueueLocal>;
* - <@LOCALFUN li_call>;
* - <@LOCALFUN li_ResolveReplyQ>;
* - <@FUNCTION li_AsyncProc>;
* - <@FUNCTION li_SyncProc>;
* - <@FUNCTION ALI_checkInputQ>;
*
*/

#ifndef _LI_PREXT_H
#define _LI_PREXT_H

/*************************************************************/
/*** PRIVATE EXTERNAL ROUTINES (LOCAL INTELLIGENCE ENGINE) ***/
/*************************************************************/

/** LI OPERATION DISPATCHER **/
/*****************************/

extern CSILONG         li_OperationDispatcher (
IN     LI_PROCCONTEXT *li_ProcContext_P,
IN     CSILONG         flags,
INOUT  LI_MSGCONTEXT  *li_MsgContext_P,
INOUT  CSIPDU         **dataDesc_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);


/** LI ALLOCATION OF A NEW MESSAGE CONTEXT **/
/********************************************/

extern CSILONG         li_AllocMsgContext (
INOUT  LI_MSGCONTEXT **li_MsgContext_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);


/** LI RELEASE OF A MESSAGE CONTEXT **/
/*************************************/

extern CSILONG         li_FreeMsgContext (
INOUT  LI_MSGCONTEXT **li_MsgContext_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);


/** LI MESSAGE ROUTING SCHEDULER **/
/**********************************/

extern CSILONG         li_MsgRoutScheduler (
IN     LI_PROCCONTEXT *li_ProcContext_P,
INOUT  LI_MSGCONTEXT  *li_MsgContext_P,
OUT    CSIPDU         *dataDesc_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI DETECTION OF A LOOPING MESSAGE (ROUTING) **/
/*************************************************/

extern CSILONG         li_LoopDetection (
IN     CSICHAR256      localGateway,
IN     CSIULONG        nbOfPrevGateways,
IN     CSICHAR        *prevGatewayBuf_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI ACCESS CONTROL MANAGEMENT SCHEDULER **/
/********************************************/

extern CSILONG         li_ACMScheduler (
IN     LI_PROCCONTEXT *li_ProcContext_P,
INOUT  LI_MSGCONTEXT  *li_MsgContext_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI DATA PRESENTATION SCHEDULER: GET CONVERSION INFORMATION **/
/****************************************************************/

extern CSILONG         li_GetConvInfo (
IN     LI_PROCCONTEXT *li_ProcContext_P,
IN     LI_MSGCONTEXT  *li_MsgContext_P,
IN     CSICHAR32       messageID,
IN     CSILONG         inputCodePage,
IN     CSICHAR128      inputHFMT,
INOUT  CSIPDU          *dataDesc_P,
OUT    CSILONG        *outputCodePage_P,
OUT    CSICHAR128      outputHFMT,
OUT    CSICHAR       **msgFormat_PP,
OUT    CSILONG        *convFlag_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI DATA PRESENTATION SCHEDULER: GET CONVERSION INFORMATION FOR VAS **/
/************************************************************************/

extern CSILONG         li_GetMsgInfoForVAS (
IN     LI_MSGCONTEXT  *li_MsgContext_P,
IN     CSICHAR32       messageID,
IN     CSILONG         inputCodePage,
IN     CSICHAR128      inputHFMT,
IN     CSILONG         outputCodePage_P,
IN     CSICHAR128      outputHFMT,
INOUT  CSIPDU          *dataDesc_P,
OUT    CSICHAR       **msgFormat_PP,
OUT    CSILONG        *convFlag_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI DATA PRESENTATION SCHEDULER: MESSAGE CONVERSION CALLER **/
/***************************************************************/

extern CSILONG         li_MsgConvertToPivot (
IN     CSICHAR        *msgFormat_P,
IN     CSICHAR32       msgId,
IN     CSICHAR128      inputHFMT,
IN     CSILONG         inputCodePage,
IN     CSILONG        *conversion_P,
INOUT  CSIPDU         **dataDesc_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

extern CSILONG         li_MsgConvertFromPivot (
IN     CSICHAR        *msgFormat_P,
IN     CSICHAR32       msgId,
IN     CSICHAR128      outputHFMT,
IN     CSILONG         outputCodePage,
IN     CSILONG        *conversion_P,
INOUT  CSIPDU         **dataDesc_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

extern CSILONG         li_MsgConvert (
IN      LI_MSGCONTEXT *li_MsgContext_P,
IN     CSICHAR        *msgFormat_P,
IN     CSICHAR32       msgId,
IN     CSICHAR128      inputHFMT,
IN     CSILONG         inputCodePage,
IN     CSICHAR128      outputHFMT,
IN     CSILONG         outputCodePage,
INOUT  CSIPDU         **dataDesc_PP,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI ORIGINAL DATA COPY: FOR REPORTS WITH DATA **/
/**************************************************/

extern CSILONG         li_SaveOriginalData (
IN     CSICHAR48       uniqueId,
IN     CSILONG         reportOptions,
INOUT  CSIDD          *dataDesc_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI ORIGINAL DATA RESTORATION: FOR REPORTS WITH DATA **/
/*********************************************************/

extern CSILONG         li_RestoreOriginalData (
IN     CSICHAR48       uniqueId,
IN     CSILONG         reportOptions,
IN     CSILONG         reportFeedback,
INOUT  CSIPDU         *dataDesc_P,
OUT    CSILONG        *returnCode_P,
OUT    CSILONG        *reasonCode_P
);

/** LI BUILD FILE NAME: FOR REPORTS WITH DATA **/
/***********************************************/

extern CSICHAR *       li_BuildFileName (
IN     CSICHAR48       uniqueId,
OUT    CSICHAR256      fileName
);

/** LI IS QUEUE LOCAL: FOR REPORTS WITH DATA **/
/**********************************************/

extern CSIBOOLEAN      li_IsQueueLocal (
IN     LI_MSGCONTEXT   *li_MsgContext_P,
IN     CSICHAR48       queueName
);

/** LI SHOULD REPORT BE DISMISSED: FOR REPORTS WITH DATA **/
/**********************************************************/

extern CSIBOOLEAN      li_ShouldReportBeDismissed (
IN     CSILONG         report,
IN     CSILONG         feedback
);

/** LI CALL: CUSTOMIZED CSI_call FUNCTION **/
/*******************************************/

extern CSILONG    li_call(
IN     CSICHAR48  svcName,
IN     CSIDD     *inDataDesc_P,
IN     CSILONG    flags,
INOUT  CSIQOS    *qos_P,
OUT    CSIDD     *outDataDesc_P,
OUT    CSILONG   *svcRetCode_P,
OUT    CSILONG   *returnCode_P,
OUT    CSILONG   *reasonCode_P
);

/** LI RESOLVEREPLYQ : REMOVE LOCAL /LOCALGW FOR REPLYQ **/
/*********************************************************/

extern CSIVOID li_ResolveReplyQ(
  IN    CSIBOOLEAN returnFQDN,
  INOUT CSICHAR48 replyQ);


extern CSILONG   li_AsyncProc(
IN        LI_PROCCONTEXT  *li_ProcContext_P,
INOUT     LI_MSGCONTEXT   *li_MsgContext_P,
OUT       CSILONG         *returnCode_P,
OUT       CSILONG         *reasonCode_P
) ;


extern CSILONG  li_SyncProc(
IN       LI_PROCCONTEXT  *li_ProcContext_P,
INOUT    CSIPDU          **request_PP,
INOUT    LI_MSGCONTEXT   *li_MsgContext_P,
OUT      CSIPDU          **reply_PP,
OUT      CSILONG         *returnCode_P,
OUT      CSILONG         *reasonCode_P,
OUT      CSILONG         *svcRetCode_P
);


extern CSILONG ALI_checkInputQ(
IN   CSIMQHCONN QMgrHandle,
IN   CSICHAR48  inputQName,
IN   CSICHAR48  svcName,
OUT  CSILONG   *returnCodeP,
OUT  CSILONG   *reasonCodeP
);

extern CSIVOID li_PrepAudit(
IN       CSILONG         lTime,
IN       CSILONG         lUTime,
INOUT    CSICHAR48       cTime
);

#endif /* _LI_PREXT_H */
/*****************************************************************************/
