/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   CCN
* @PACKAGE  LI (Local Intelligence engine)
*
* @FILE     li_prmes.h
*
* @ABSTRACT
* Local Intelligence PRivate MESsages definitions (Logging system).
* This file contains:
* - the definitions of the logging messages, be it trace, error, system or
*   audit logs.
*
*/

#ifndef _LI_PRMES_H
#define _LI_PRMES_H

#include "evr_upmac.h"

/********************/
/*** ERROR EVENTS ***/
/********************/

/** SEVERITY: UNDETERMINATE **/
/*****************************/

#define LI_MSG_OS_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,1)
EVR_SET_STRING(LI_MSG_OS_ERROR,
   "ERROR: The OS library %s verb failed, reasonCode %d")

#define LI_MSG_PRES_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,2)
EVR_SET_STRING(LI_MSG_PRES_ERROR,
   "ERROR: The PRES library %s verb failed, reasonCode %d")

#define LI_MSG_GCN_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,3)
EVR_SET_STRING(LI_MSG_GCN_ERROR,
   "ERROR: The GCN library %s verb failed, reasonCode %d")

#define LI_MSG_CSI_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,4)
EVR_SET_STRING(LI_MSG_CSI_ERROR,
   "ERROR: The CSI library %s verb failed, reasonCode %d")

#define LI_MSG_CDX_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,5)
EVR_SET_STRING(LI_MSG_CDX_ERROR,
   "ERROR: The CDX api %s verb failed, reasonCode %d")

#define LI_MSG_ROU_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,6)
EVR_SET_STRING(LI_MSG_ROU_ERROR,
   "ERROR: The ROU module %s function failed, reasonCode %d")

#define LI_MSG_PRV_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,7)
EVR_SET_STRING(LI_MSG_PRV_ERROR,
   "ERROR: The PRV library %s function failed, reasonCode %d")

#define LI_MSG_ACM_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,8)
EVR_SET_STRING(LI_MSG_ACM_ERROR,
   "ERROR: The ACM library %s function failed, reasonCode %d")

#define LI_MSG_RES_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,9)
EVR_SET_STRING(LI_MSG_RES_ERROR,
   "ERROR: %s in %s")

#define LI_MSG_CDX_GET_INTERNAL_SYNTAX \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_UNDETERMINATE,10)
EVR_SET_STRING(LI_MSG_CDX_GET_INTERNAL_SYNTAX,
   "ERROR: Cannot retrieve internal syntax for message: %s, reasonCode %d")


/** SEVERITY: CRITICAL **/
/************************/

#define LI_MSG_NULL_POINTER \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_CRITICAL,11)
EVR_SET_STRING(LI_MSG_NULL_POINTER,
   "CRITICAL ERROR: The pointer %s is NULL")

#define LI_MSG_INVAL_PARAM \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_CRITICAL,12)
EVR_SET_STRING(LI_MSG_INVAL_PARAM,
   "CRITICAL ERROR: The parameter %s is invalid")

#define LI_MSG_MQ_OPEN_FAILED  \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_CRITICAL,13)
EVR_SET_STRING(LI_MSG_MQ_OPEN_FAILED,
   "CRITICAL ERROR: Cannot open the %s queue [return code = %ld] [reason code = %ld].")

/** SEVERITY: MAJOR **/
/*********************/

#define LI_MSG_INIT_OS_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 21)
EVR_SET_STRING(LI_MSG_INIT_OS_ERROR,
   "MAJOR ERROR: Cannot init the OS module, reason = [%d]")

#define LI_MSG_INIT_CDX_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 22)
EVR_SET_STRING(LI_MSG_INIT_CDX_ERROR,
   "MAJOR ERROR: Cannot bind with the CDX module, reason = [%d]")

#define LI_MSG_INIT_PRES_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 23)
EVR_SET_STRING(LI_MSG_INIT_PRES_ERROR,
   "MAJOR ERROR: Cannot bind with the PRES module, reason = [%d]")

#define LI_MSG_INIT_CSI_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 24)
EVR_SET_STRING(LI_MSG_INIT_CSI_ERROR,
   "MAJOR ERROR: Cannot bind with the CSI layer, reason = [%d]")

#define LI_MSG_TRGQ_OPEN_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 25)
EVR_SET_STRING(LI_MSG_TRGQ_OPEN_ERROR,
   "MAJOR ERROR: The triggering queue %s cannot be opened, reason = [%d]")

#define LI_MSG_NEXTQ_PUT_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 26)
EVR_SET_STRING(LI_MSG_NEXTQ_PUT_ERROR,
   "MAJOR ERROR: The aynchronous message cannot be put in the queue %s, \
reason = [%d]")

#define LI_MSG_SWITCH_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 27)
EVR_SET_STRING(LI_MSG_SWITCH_ERROR,
   "MAJOR ERROR: Unable to switch message [un.id = %s] to degraded mode, \
reason = [%d]")

#define LI_MSG_CALL_FAILED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 28)
EVR_SET_STRING(LI_MSG_CALL_FAILED,
   "MAJOR ERROR: The call to %s service [%s] failed, reason = [%d]")

#define LI_MSG_DEGRQ_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 29)
EVR_SET_STRING(LI_MSG_DEGRQ_ERROR,
   "MAJOR ERROR: The degraded message cannot be put in the queue %s, \
reason = [%d]")

#define LI_MSG_LOOP_DETECTED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 30)
EVR_SET_STRING(LI_MSG_LOOP_DETECTED,
   "MAJOR ERROR: The current message is looping ! (routing problem)")

#define LI_MSG_NOT_CONV \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 31)
EVR_SET_STRING(LI_MSG_NOT_CONV,
   "MAJOR ERROR: The conversational service was not reached by CSI_connect")

/** SEVERITY: MINOR **/
/*********************/

#define LI_MSG_ENV_VAR_ERROR \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MINOR, 44)
EVR_SET_STRING(LI_MSG_ENV_VAR_ERROR,
   "MINOR ERROR: Cannot find the environment variable %s")

#define LI_MSG_DEGR_SWITCH \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MINOR, 45)
EVR_SET_STRING(LI_MSG_DEGR_SWITCH,
   "MINOR ERROR: The message (un.id = [%s]) switched to degraded mode")

#define LI_MSG_DEAD_QUEUE_PUT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 46)
EVR_SET_STRING(LI_MSG_DEAD_QUEUE_PUT,
   "MAJOR ERROR: Message has been put in the dead letter queue for reason: %d")

#define LI_MSG_MQ_INQUIRE_FAILED  \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MINOR,47)
EVR_SET_STRING(LI_MSG_MQ_INQUIRE_FAILED,
   "MINOR ERROR: Inquiring the %s queue failed [reason code = %ld].")

#define LI_MSG_MQ_SET_FAILED  \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MINOR,48)
EVR_SET_STRING  (LI_MSG_MQ_SET_FAILED,
   "MINOR ERROR: Setting the %s queue failed [ reason code = %ld].")

#define LI_MSG_DISCARDED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_MAJOR, 49)
EVR_SET_STRING(LI_MSG_DISCARDED,
   "MAJOR ERROR: Message has been discarded for reason: %d")

/** SEVERITY: USER ERROR **/
/**************************/

#define LI_MSG_ACCESS_DENIED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_USERERROR, 60)
EVR_SET_STRING(LI_MSG_ACCESS_DENIED,
   "USER ERROR: Access Denied to user %s for resource %s")

#define LI_MSG_BAD_REPLY_TO_Q \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_USERERROR, 61)
EVR_SET_STRING(LI_MSG_BAD_REPLY_TO_Q,
   "USER ERROR: A report with data is required for message [%s], but \
the ReplyToQ [ %s ] is remote or invalid => report with no data.")

#define LI_MSG_EMPTY_RESOURCE_NAME \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_UNDEFINED, EVR_USERERROR, 62)
EVR_SET_STRING(LI_MSG_EMPTY_RESOURCE_NAME,
   "USER ERROR: Impossible to query directory: no resource given")

/**************************************/
/*** DEFINITION OF THE TRACE EVENTS ***/
/**************************************/

/** Trace Level: EVR_COMPONENT **/
/********************************/


/** Trace Level: EVR_DEBUG **/
/****************************/

#define LI_TRC_TRIGGER_EVENT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 111)
EVR_SET_STRING(LI_TRC_TRIGGER_EVENT,
   "INFO: Trigger message detected. Trigger queue : [%s]")

#define LI_TRC_SYNC_EVENT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 112)
EVR_SET_STRING(LI_TRC_SYNC_EVENT,
"INFO: Synchronous message detected.\n \
Unique Id: [%s].\n \
Origin: user [%s] from appli [%s].\n \
Target: service [%s]")

/* obsolete degraded mode event */
#define LI_TRC_DEGRADED_EVENT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 113)
EVR_SET_STRING(LI_TRC_DEGRADED_EVENT,
"INFO: Degraded-Mode message detected.\n \
Unique Id: [%s].\n \
Origin: user [%s] from appli [%s]. \n \
Target: service [%s]")

#define LI_TRC_SVC_CALLED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 114)
EVR_SET_STRING(LI_TRC_SVC_CALLED,
"INFO: The synchronous message (un.id = [%s]) was transmitted to \
 service %s")

#define LI_TRC_PUT_IN_Q \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 115)
EVR_SET_STRING(LI_TRC_PUT_IN_Q,
"INFO: The asynchronous message (un.id = [%s]) was put in the queue %s")

#define LI_TRC_CONV_EVENT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 116)
EVR_SET_STRING(LI_TRC_CONV_EVENT,
"INFO: Connection detected.\n \
Unique Id: [%s].\n \
Origin: user [%s] from appli [%s].\n \
Target: conversational service [%s].\n \
CD_init: [%d].\n \
Flags: [%x]")

#define LI_TRC_CONNECT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 117)
EVR_SET_STRING(LI_TRC_CONNECT,
"INFO: The LI connected service [%s], CD_sub: [%d]")

#define LI_TRC_MSG_RECVED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 118)
EVR_SET_STRING(LI_TRC_MSG_RECVED,
"INFO: Receiving... From = [%d]")

#define LI_TRC_MSG_SENT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 119)
EVR_SET_STRING(LI_TRC_MSG_SENT,
"INFO: Sending... To = [%d]")

#define LI_TRC_SWAP_TO_FROM \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 120)
EVR_SET_STRING(LI_TRC_SWAP_TO_FROM,
"INFO: To and From swapped")

#define LI_TRC_ACCESS_GRANTED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 121)
EVR_SET_STRING(LI_TRC_ACCESS_GRANTED,
"INFO: Access Granted to user %s for resource %s")

#define LI_TRC_CSI_NO_MSG \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 122)
EVR_SET_STRING(LI_TRC_CSI_NO_MSG,
"INFO: No more message in input queue")

#define LI_TRC_ORIGINAL_DATA_SAVED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 123)
EVR_SET_STRING(LI_TRC_ORIGINAL_DATA_SAVED,
"INFO: %d bytes of data saved into file [ %s ].")

#define LI_TRC_ORIGINAL_DATA_RESTORED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 124)
EVR_SET_STRING(LI_TRC_ORIGINAL_DATA_RESTORED,
"INFO: %d bytes of data restored from file [ %s ].")

#define LI_TRC_ORIGINAL_DATA_DELETED \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 125)
EVR_SET_STRING(LI_TRC_ORIGINAL_DATA_DELETED,
"INFO: Data delivered, file [ %s ] deleted.")

#define LI_TRC_CHECK_INPUT_QUEUE \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 126)
EVR_SET_STRING(LI_TRC_CHECK_INPUT_QUEUE,
   "PROCESS TRACE: Checking if messages are available in the input queue [name = %s].")

#define LI_TRC_MQINQ_RESULT \
EVR_SET_CODE(EVR_LI_COMPONENT, EVR_DEBUG, EVR_UNDEFINED, 127)
EVR_SET_STRING(LI_TRC_MQINQ_RESULT,
 "MQInquire results: process max number: %ld current depth: %ld.")

#endif /* _LI_PRMES_H */
/*****************************************************************************/
