/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   CCN
* @PACKAGE  LI (Local Intelligence engine)
*
* @FILE     li_prssc.c
*
* @ABSTRACT
* The Local Intelligence data PReSentation SCheduler.
* This file contains:
*
* @INCLUDE
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*** COMMON DEFINITIONS AND CONSTANTS INCLUDE FILES ***/
/******************************************************/
#include "gcn_upcon.h"
#include "gcn_uptyp.h"

/*** OS INCLUDE FILES ***/
/************************/
#include "osx_upcon.h"
#include "osx_upext.h"

/*** LOGGING SYSTEM INCLUDE FILES ***/
/************************************/
#include "loglo_uptyp.h"
#include "evr_upext.h"
#include "evr_upmac.h"
#include "gcn_upmes.h"
#include "li_prmes.h"
#include "gcn_upmac.h"

/*** CCN DIRECTORY eXtended ACCESS INCLUDE FILES ***/
/***************************************************/
#include "cdx_upcon.h"
#include "cdx_uptyp.h"
#include "cdx_upext.h"

/*** COMMON TOOLS INCLUDE FILES ***/
/**********************************/
#include "gcn_upext.h"

/*** PRESENTATION INCLUDE FILES ***/
/**********************************/
#include "prs_upcon.h"
#include "prs_upext.h"

/*** MESSAGE CONVERSION INCLUDE FILES ***/
/****************************************/
#include "prv_upcon.h"
#include "prv_usext.h"
#include "csc_upcon.h"

/*** LOCAL INTELLIGENCE INCLUDE FILES ***/
/****************************************/
#include "li_prcon.h"
#include "li_prtyp.h"
#include "li_prext.h"
#include "li_upext.h"

/****************************************/
/*** LI GET (MESSAGE) CONVERSION INFO ***/
/****************************************/
/*
* @LOCALFUN li_GetConvInfo
*
* @DESCRIPTION Gets the conversion info in the directory
*
* @ABSTRACT
* This functions determines the necessary conversions the message will have to
* undergo. It also returns the conversions parameters. It relies on the CDX
* functions <@FUNCTION CDX_getInternalSyntaxOfMsg> (internal syntax of the
* message), <@FUNCTION CDX_getMsgFormatRulesOfResource> (To get the HFMT and
* the Code Page of target host), as well as presentation function
* <@FUNCTION PRES_fmtcomp> to compare initial and required final formats.
*
* This function is called by <@LOCALFUN li_OperationDispatcher>.
*
* @RETURNVALUE CSILONG
* Possible returnvalues are : CSISUCCESS (normal) or CSIFAILED (error).
*
* @PARAMETER
* IN     LI_PROCCONTEXT *li_ProcContext_P
*        Points to the Local Intelligence process context.
*
* IN     LI_MSGCONTEXT *li_MsgContext_P
*        Points to the LI current processed message context.
*
* IN     CSICHAR32 messageID
*        The message TYPE identifier. (The one extracted by PRES_gettype).
*
* IN     CSILONG inputCodePage
*        The initial CodePage of the message.
*
* IN     CSICHAR128 inputHFMT
*        The initial Host Format of the message.
*
* INOUT  CSIPDU *dataDesc_P
*        Pointeur to the concerned Data Descriptor.
*
* OUT    CSILONG *outputCodePage_P
*        The retrieved destination Code Page.
*
* OUT    CSICHAR128 outputHFMT
*        The retrieved Host Format of the destination application.
*
* OUT    CSICHAR **msgFormat_P
*        The retrieved Internal Syntax of the message (also called XFMT).
*
* OUT    CSILONG *convFlag_P
*        Possible values TRUE or FALSE. Flag indicating the necessity of a
*        conversion (if input and output formats are not compatible).
*
* OUT    CSILONG *returnCode_P
*        Points to the returnCode (CSISUCCESS or  CSIFAILED).
*
* OUT    CSILONG *reasonCode_P
*        Normally points to the value LI_ERR_NONE. In case of error, points
*        to the specific error code.
*
* @SEEALSO
* <@FUNCTION PRES_fmtcomp>
*/
CSILONG li_GetConvInfo(
IN      LI_PROCCONTEXT *li_ProcContext_P,
IN      LI_MSGCONTEXT  *li_MsgContext_P,
IN      CSICHAR32       messageID,
IN      CSILONG         inputCodePage,
IN      CSICHAR128      inputHFMT,
INOUT   CSIPDU          *dataDesc_P,
OUT     CSILONG        *outputCodePage_P,
OUT     CSICHAR128      outputHFMT,
OUT     CSICHAR       **msgFormat_PP,
OUT     CSILONG        *convFlag_P,
OUT     CSILONG        *returnCode_P,
OUT     CSILONG        *reasonCode_P
)
{
   /*** LOCAL VARIABLES DECLARATION ***/
   /***********************************/

   CSILONG      ret    = CSISUCCESS;
   CSILONG      status = LI_ERR_NONE;

   /** Variables for the Directory (eXtended) Access **/

   CDXRESULT    cdxMsgFormat_S = {CDXRESULT_DEFAULT};
                                       /* Structure retrieved through CDX  */
   CSICHAR     *tmpMsgFmtPtr_P = NULL; /* Temporary Message Format Pointer */
   CSICHAR128   tmpOutputHFMT  = {""}; /* Temporary output Host Format...  */

   CSIULONG     fmtOSlen       = 0L;   /* OS_mem_alloc'ated size of buffer */
   CSIULONG     cdxLen         = 0L;   /* Length of retrieved format       */

   /** Variables associated with the Message Conversion **/

   CSILONG      response = FALSE;      /* Boolean for the PRES_fmtcomp verb */

   CSICHAR48    resourceName   = {""};

   TRACE_FUNCTION_ENTRY("li_GetConvInfo");

   /*** PARAMETERS VERIFICATION ***/
   /*******************************/

   if ( (NULL==returnCode_P)||(NULL==reasonCode_P) )
   {
      TRACE_FUNCTION_EXIT("li_GetConvInfo",CSIFAILED,LI_ERR_INTERNAL);
      return(CSIFAILED);
   }

   *returnCode_P = CSISUCCESS;
   *reasonCode_P = LI_ERR_NONE;

   /*** INITIALISATIONS ***/
   /***********************/

   if (LI_ERR_NONE == status)
   {
      *convFlag_P = FALSE;
   }

   /** Get Size of OS_mem_alloc'ated *msgFormat_PP buffer **/

   if (LI_ERR_NONE == status)
   {
      ret = OS_mem_getsize( (*msgFormat_PP), &fmtOSlen, 
            returnCode_P, reasonCode_P);
      if (CSIFAILED==ret)
      {
         EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_OS_ERROR,
         "OS_mem_getsize",(*reasonCode_P)));
         status = LI_ERR_OS_MEM_GETSIZE;
      }
   }


   /*** STORE USEFUL INFO IN CONTEXT, IF NEED BE ***/
   /************************************************/

   if (LI_ERR_NONE == status)
   {
      if ('\0' == li_MsgContext_P->ClientHostFormat[0])
      {
         strncpy(li_MsgContext_P->ClientHostFormat, inputHFMT,
                 sizeof(CSICHAR128));
         li_MsgContext_P->ClientCodePage = inputCodePage;
      }
   }

   /*** RETRIEVAL OF THE DIRECTORY INFORMATION ***/
   /**********************************************/

   /** Retrieving the XFMT(internal syntax) of the current message **/

    if (LI_ERR_NONE == status)
    {
     ret = CDX_getInternalSyntaxOfMsg(messageID, &cdxMsgFormat_S,
           returnCode_P,reasonCode_P);
     if (CSIFAILED == ret)
        {
            EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_CDX_GET_INTERNAL_SYNTAX,
            messageID,(*reasonCode_P)));
            status = LI_ERR_CDX_GET_SYNTAX;
        }
     }
 
    /** Converting the CDXRESULT to the right type (CSICHAR*) **/

     if (LI_ERR_NONE == status)
     {
        tmpMsgFmtPtr_P = (CSICHAR *) CDX_list_first(NULL,&cdxMsgFormat_S);
        if (NULL!=tmpMsgFmtPtr_P)
        {
            cdxLen = (CSIULONG) strlen(tmpMsgFmtPtr_P) + 1L;

            if (fmtOSlen < cdxLen)
            {
              ret = OS_mem_realloc(cdxLen,(CSIVOID**)msgFormat_PP,
                    returnCode_P,reasonCode_P);
              if (CSIFAILED==ret)
              {
                 EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_OS_ERROR,
                 "OS_mem_realloc",(*reasonCode_P)));
                 status = LI_ERR_ALLOC;
              }
            }

            if (LI_ERR_NONE==status)
            {
               memcpy((*msgFormat_PP), tmpMsgFmtPtr_P, cdxLen);        
            }
        }
      }
      CDX_free(NULL,&cdxMsgFormat_S);

      /*** RETRIEVING THE HFMT AND CODEPAGE OF THE TARGET APPLICATION ***/
      /******************************************************************/

      if (LI_ERR_NONE == status)
      {
           if ('\0' == li_MsgContext_P->ServerHostFormat[0])
           {   /* Retrieve the information from the Directory... */
               if (CSI_SYNCHRON==li_MsgContext_P->ExchangeMode)
               {
                  strncpy(resourceName, 
                          li_MsgContext_P->TraceInfo_P->TrgtCCNServiceName,
                          sizeof(CSICHAR48));
               }
               else
               {
                  strncpy(resourceName,
                          li_MsgContext_P->TraceInfo_P->TrgtCCNQueueName,
                          sizeof(CSICHAR48));
               }

               ret = CDX_getMsgFormatRulesOfResource(li_MsgContext_P->DestGateway,
                           li_MsgContext_P->ExchangeMode,
                           resourceName, &tmpOutputHFMT, outputCodePage_P,
                           returnCode_P, reasonCode_P);
               if (CSIFAILED == ret)
               {
                  status = *reasonCode_P;
               }
               else
               {
                  memcpy(outputHFMT, tmpOutputHFMT, sizeof(CSICHAR128));
               }
      
               /* Retrieved information is copied into the Msg Context */
      
               strncpy(li_MsgContext_P->ServerHostFormat, outputHFMT,
                       sizeof(CSICHAR128) );
                       li_MsgContext_P->ServerCodePage = *outputCodePage_P;
           }
           else
           {   /* Retrieve info from context, depending on Direction field */
               switch(li_MsgContext_P->Direction)
               {
                case LI_REQUEST:
                /* Probably occurs only in Conversational Mode, but who knows ?... */
                strncpy(outputHFMT, li_MsgContext_P->ServerHostFormat,
                        sizeof(CSICHAR128));
                *outputCodePage_P = li_MsgContext_P->ServerCodePage;
                break;

                case LI_REPLY:
                /* For instance reply DD of a CSI_call... */
                strncpy(outputHFMT, li_MsgContext_P->ClientHostFormat,
                        sizeof(CSICHAR128));
                *outputCodePage_P = li_MsgContext_P->ClientCodePage;
                break;

               default:
               status = LI_ERR_INTERNAL;
               break;
              }
           }
      }
   /*** CHECKING THE COMPATIBILITY OF THE FORMATS ***/
   /*************************************************/

   if (LI_ERR_NONE == status)
   {
      ret = PRES_fmtcomp(dataDesc_P, *msgFormat_PP, inputHFMT, outputHFMT,
            inputCodePage, *outputCodePage_P, &response,
            returnCode_P, reasonCode_P);
      if (CSIFAILED==ret)
      {
         EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_PRV_ERROR,
         "PRES_fmt_comp",(*reasonCode_P)));
         status = LI_ERR_PRES_FMTCOMP;
      }

      *convFlag_P = response;
   }

   /*** THE END ***/
   /***************/

   if (LI_ERR_NONE != status) *returnCode_P = CSIFAILED;
                         else *returnCode_P = CSISUCCESS;

   *reasonCode_P = status;

   TRACE_FUNCTION_EXIT("li_GetConvInfo",*returnCode_P,*reasonCode_P);
   return(*returnCode_P);
}


/*************************************************************/
/*** LI GET MESSAGE (CONVERSION) INFO FOR VAS (PROCESSING) ***/
/*************************************************************/
/*
* @LOCALFUN li_GetMsgInfoForVAS
*
* @DESCRIPTION Gets VAS conversion info in the directory
*
* @ABSTRACT
* This functions determines the necessary conversions the message will have to
* undergo. It also returns the conversions parameters.
*
* @RETURNVALUE CSILONG
* Possible returnvalues are : CSISUCCESS (normal) or CSIFAILED (error).
*
* @PARAMETER
* IN     LI_PROCCONTEXT *li_ProcContext_P
*        Points to the Local Intelligence process context.
*
* IN     LI_MSGCONTEXT *li_MsgContext_P
*        Points to the LI current processed message context.
*
* IN     CSICHAR32 messageID
*        The message TYPE identifier. (The one extracted by PRES_gettype).
*
* IN     CSILONG inputCodePage
*        The initial CodePage of the message.
*
* IN     CSICHAR128 inputHFMT
*        The initial Host Format of the message.
*
* INOUT  CSIPDU *dataDesc_P
*        Pointeur to the concerned Data Descriptor.
*
* OUT    CSILONG *outputCodePage_P
*        The retrieved destination Code Page.
*
* OUT    CSICHAR128 outputHFMT
*        The retrieved Host Format of the destination application.
*
* OUT    CSICHAR **msgFormat_P
*        The retrieved Internal Syntax of the message (also called XFMT).
*
* OUT    CSILONG *convFlag_P
*        Possible values TRUE or FALSE. Flag indicating the necessity of a
*        conversion (if input and output formats are not compatible).
*
* OUT    CSILONG *returnCode_P
*        Points to the returnCode (CSISUCCESS or  CSIFAILED).
*
* OUT    CSILONG *reasonCode_P
*        Normally points to the value LI_ERR_NONE. In case of error, points
*        to the specific error code.
*
* @SEEALSO
* <@FUNCTION PRES_fmtcomp>
*/
CSILONG li_GetMsgInfoForVAS(
IN      LI_MSGCONTEXT  *li_MsgContext_P,
IN      CSICHAR32       messageID,
IN      CSILONG         inputCodePage,
IN      CSICHAR128      inputHFMT,
IN      CSILONG         outputCodePage,
IN      CSICHAR128      outputHFMT,
INOUT   CSIPDU          *dataDesc_P,
OUT     CSICHAR       **msgFormat_PP,
OUT     CSILONG        *convFlag_P,
OUT     CSILONG        *returnCode_P,
OUT     CSILONG        *reasonCode_P
) {
  /*** LOCAL VARIABLES DECLARATION ***/
  /***********************************/

  CSILONG      ret    = CSISUCCESS;
  CSILONG      status = LI_ERR_NONE;

  /** Variables for the Directory (eXtended) Access **/

  CDXRESULT    cdxMsgFormat_S = {CDXRESULT_DEFAULT};
                                      /* Structure retrieved through CDX  */
  CSICHAR     *tmpMsgFmtPtr_P = NULL; /* Temporary Message Format Pointer */

  CSIULONG     fmtOSlen       = 0L;   /* OS_mem_alloc'ated size of buffer */
  CSIULONG     cdxLen         = 0L;   /* Length of retrieved format       */

  /** Variables associated with the Message Conversion **/

  CSILONG      response = FALSE;      /* Boolean for the PRES_fmtcomp verb */

  TRACE_FUNCTION_ENTRY("li_GetMsgInfoForVAS");

  /*** PARAMETERS VERIFICATION ***/
  /*******************************/

  if ( (NULL==returnCode_P)||(NULL==reasonCode_P) )  {
    TRACE_FUNCTION_EXIT("li_GetMsgInfoForVAS",CSIFAILED,LI_ERR_INTERNAL);
    return(CSIFAILED);
  }

  *returnCode_P = CSISUCCESS;
  *reasonCode_P = LI_ERR_NONE;

  /*** INITIALISATIONS ***/
  /***********************/

  if (LI_ERR_NONE == status)
    *convFlag_P = FALSE;

  /** Get Size of OS_mem_alloc'ated *msgFormat_PP buffer **/

  if (LI_ERR_NONE == status) {
    ret = OS_mem_getsize( (*msgFormat_PP), &fmtOSlen, 
                          returnCode_P, reasonCode_P);
    if (CSIFAILED==ret) {
      EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_OS_ERROR,
                                      "OS_mem_getsize",(*reasonCode_P)));
      status = LI_ERR_OS_MEM_GETSIZE;
    }
  }


  /*** STORE USEFUL INFO IN CONTEXT, IF NEED BE ***/
  /************************************************/

  if (LI_ERR_NONE == status) {
    if ('\0' == li_MsgContext_P->ClientHostFormat[0]) {
      strncpy(li_MsgContext_P->ClientHostFormat, inputHFMT,
              sizeof(CSICHAR128));
      li_MsgContext_P->ClientCodePage = inputCodePage;
    }
  }

  /*** RETRIEVAL OF THE DIRECTORY INFORMATION ***/
  /**********************************************/

  /** Retrieving the XFMT(internal syntax) of the current message **/

  if (LI_ERR_NONE == status) {
    ret = CDX_getInternalSyntaxOfMsg(messageID, &cdxMsgFormat_S,
                                     returnCode_P,reasonCode_P);
    if (CSIFAILED == ret) {
      EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_CDX_GET_INTERNAL_SYNTAX,
                                      messageID,
                                      (*reasonCode_P)));
      status = LI_ERR_CDX_GET_SYNTAX;
    }
  }
 
  /** Converting the CDXRESULT to the right type (CSICHAR*) **/

  if (LI_ERR_NONE == status) {
    tmpMsgFmtPtr_P = (CSICHAR *) CDX_list_first(NULL,&cdxMsgFormat_S);
    if (NULL!=tmpMsgFmtPtr_P) {
      cdxLen = (CSIULONG) strlen(tmpMsgFmtPtr_P) + 1L;

      if (fmtOSlen < cdxLen) {
        ret = OS_mem_realloc(cdxLen,(CSIVOID**)msgFormat_PP,
                             returnCode_P,reasonCode_P);
        if (CSIFAILED==ret) {
          EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_OS_ERROR,
                                          "OS_mem_realloc",(*reasonCode_P)));
          status = LI_ERR_ALLOC;
        }
      }

      if (LI_ERR_NONE==status)
        memcpy((*msgFormat_PP), tmpMsgFmtPtr_P, cdxLen);        
    }
  }
  CDX_free(NULL,&cdxMsgFormat_S);

  /*** CHECKING THE COMPATIBILITY OF THE FORMATS ***/
  /*************************************************/

  if (LI_ERR_NONE == status) {
    ret = PRES_fmtcomp(dataDesc_P, *msgFormat_PP, inputHFMT, outputHFMT,
                       inputCodePage, outputCodePage, &response,
                       returnCode_P, reasonCode_P);
    if (CSIFAILED==ret) {
      EVR_ERRORLOG(EVR_UNDETERMINATE,(LI_MSG_PRV_ERROR,
                                      "PRES_fmt_comp",(*reasonCode_P)));
      status = LI_ERR_PRES_FMTCOMP;
    }

    *convFlag_P = response;
  }

  /*** THE END ***/
  /***************/

  if (LI_ERR_NONE != status)
    *returnCode_P = CSIFAILED;
  else 
    *returnCode_P = CSISUCCESS;

  *reasonCode_P = status;

  TRACE_FUNCTION_EXIT("li_GetMsgInfoForVAS",*returnCode_P,*reasonCode_P);
  return(*returnCode_P);
}

/******************************/
/*** LI: MESSAGE CONVERSION ***/
/******************************/
/*
* @LOCALFUN li_MsgConvertToPivot
*
* @DESCRIPTION Converts message from format A into format pivot
*
* @ABSTRACT
* This routine relies on <@FUNCTION PRES_native2pivot> to convert the message
* from pivot format to target format.
* It applies the  following algorithm:
*   Compares initial format with pivot using <@FUNCTION PRES_fmtcomp>
*   If different,
*     Converts message to pivot into a temporary data descriptor, 
*     using <@FUNCTION PRES_native2pivot>
*     If both input and temporary data descriptors are not null
*       Deletes the file referenced by input data descriptor using
*       <@FUNCTION gcn_DeleteFile>
*       If a conversion has already been performed in current LI service,
*         Releases memory used by input data descriptor using
*         <@FUNCTION gcn_FreeDataDesc>
*       Replaces input data descriptor by temporary data descriptor
*       allocated by <@FUNCTION PRES_native2pivot>
* The pivot format is (HFMT Pivot, CodePage UNICODE).
*
* @RETURNVALUE CSILONG
* Possible returnvalues are: CSISUCCESS (normal) or CSIFAILED (error).
*
* @PARAMETER
* IN    CSICHAR *msgFormat_P
*       The internal syntax of the message (a.k.a XFMT), as previously
*       retrieved in the Directory.
*
* IN    CSICHAR32 msgId
*       The message type identifier.
*
* IN    CSICHAR128 inputHFMT
*       The initial HostFormat of the message.
*
* IN    CSILONG inputCodePage
*       The initial CodePage of the message.
*
* IN    CSILONG *conversion_P
*       If TRUE, the memory used by input data descriptor should be released.
*       Input data descriptor should not be freed if it has never been converted
*       in current LI service. Data descriptor allocated at LI service entrance
*       should be freed through GTX_return.
*
* INOUT CSIPDU **dataDesc_PP
*       The Data Descriptor which is converted.
*
* OUT   CSILONG returnCode_P
*       Result of the operation (same as the function return code)
*
* OUT   CSILONG reasonCode_P
*       Reason why the operation fails (if it fails)
*       If no error occurred, it is LI_ERR_NONE.
*
* @SEEALSO
* <@FUNCTION PRES_native2pivot>
* <@FUNCTION PRES_pivot2native>
* <@FUNCTION PRES_fmtcomp>
*/
CSILONG li_MsgConvertToPivot(
IN      CSICHAR       *msgFormat_P,
IN      CSICHAR32      msgId,
IN      CSICHAR128     inputHFMT,
IN      CSILONG        inputCodePage,
IN      CSILONG       *conversion_P,
INOUT   CSIPDU        **dataDesc_PP,
OUT     CSILONG       *returnCode_P,
OUT     CSILONG       *reasonCode_P
)
{
   /*** LOCAL VARIABLES DECLARATIONS ***/
   /************************************/

   CSILONG retCode        = CSISUCCESS;
   CSILONG reaCode        = CSI_ERR_NONE;
   CSILONG status         = LI_ERR_NONE;
   CSILONG middleCodePage = LI_UNICODE;
   CSILONG response       = FALSE;
   CSIPDU *tmpDataDesc_P  = NULL;


   TRACE_FUNCTION_ENTRY("li_MsgConvertToPivot");

   /*** PARAMETERS VERIFICATION ***/
   /*******************************/

   if ( (NULL == returnCode_P) || (NULL == reasonCode_P) )
   {
      TRACE_FUNCTION_EXIT("li_MsgConvertToPivot", CSIFAILED, LI_ERR_INTERNAL);
      return(CSIFAILED);
   }

   *returnCode_P = CSISUCCESS;
   *reasonCode_P = LI_ERR_NONE;

   /*** COMPARE INITIAL FORMAT WITH PIVOT ***/
   /*****************************************/

   if (LI_ERR_NONE == status)
   { /** BEGIN: IF OK 1 **/

     PRES_fmtcomp(*dataDesc_PP, msgFormat_P, inputHFMT, PRV_PIVOT_HFMT, 
                  inputCodePage, middleCodePage, &response, 
                  returnCode_P, reasonCode_P);
     if (CSIFAILED == *returnCode_P)
     {
       EVR_ERRORLOG(EVR_UNDETERMINATE,
                    (LI_MSG_PRV_ERROR, "PRES_fmtcomp",(*reasonCode_P)));
       status = LI_ERR_PRES_FMTCOMP;
     }
   } /** ENDOF: IF OK 1 **/

   /*** IF INPUT HFMT IS NOT PIVOT, CONVERT TO PIVOT ***/
   /****************************************************/
   
   if ( (LI_ERR_NONE == status) && (TRUE == response) )
   { /** BEGIN: IF OK 2 **/

     PRES_native2pivot(msgFormat_P, msgId, inputHFMT, inputCodePage,
                       &middleCodePage, *dataDesc_PP, &tmpDataDesc_P,
                       returnCode_P, reasonCode_P);
     if (CSIFAILED == *returnCode_P)
     {
        EVR_ERRORLOG(EVR_UNDETERMINATE,
                     (LI_MSG_PRV_ERROR, "PRES_native2pivot",(*reasonCode_P)));
        status = LI_ERR_PRES_NATIVE2PIVOT;
     }

     /** Free dataDesc_PP data descriptor if necessary and **/
     /** replace it by tmpDataDesc_P                       **/
     /*******************************************************/
     if ( (NULL != tmpDataDesc_P) && (NULL != *dataDesc_PP) )
     {
       /* RFA-145 : pass run mode to temporary CSIDD */
       tmpDataDesc_P->Flags |= ((*dataDesc_PP)->Flags & CSI_DD_FLAGS_MODE_SET);

       /* Delete File if no more usefull */
       gcn_DeleteFile(*dataDesc_PP, &retCode, &reaCode);

       /* Release memory used by input data descriptor if a conversion has */
       /* already occured in the LI service */
       if (*conversion_P == TRUE)
       {
          gcn_FreeDataDesc(dataDesc_PP, returnCode_P, reasonCode_P);
          if (CSIFAILED == *returnCode_P)
          {
            EVR_ERRORLOG(EVR_UNDETERMINATE,
                         (LI_MSG_GCN_ERROR, "gcn_FreeDataDesc",(*reasonCode_P)));
            status = LI_ERR_INTERNAL;
          }
       }
       *conversion_P = TRUE;
       *dataDesc_PP = tmpDataDesc_P;
     }
   } /** ENDOF: IF OK 2 **/

   /*** THE END ***/
   /***************/

   if (LI_ERR_NONE != status) 
     *returnCode_P = CSIFAILED;
   else 
     *returnCode_P = CSISUCCESS;
   *reasonCode_P = status;

   TRACE_FUNCTION_EXIT("li_MsgConvertToPivot", *returnCode_P, *reasonCode_P);

   return(*returnCode_P);
}

/************************************************/
/*** LI: MESSAGE CONVERSION FROM FORMAT PIVOT ***/
/************************************************/
/*
* @LOCALFUN li_MsgConvertFromPivot
*
* @DESCRIPTION Converts message from format pivot into format B
*
* @ABSTRACT
* This routine relies on <@FUNCTION PRES_pivot2native> to convert the message 
* from pivot format to target format. 
* It applies the  following algorithm:
*   Compares required format with pivot using <@FUNCTION PRES_fmtcomp>
*   If different,
*     Converts message from pivot to required format into a temporary data
*     descriptor, using <@FUNCTION PRES_pivot2native>
*     If both input and temporary data descriptors are not null
*       Deletes the file referenced by input data descriptor using 
*       <@FUNCTION gcn_DeleteFile>
*       If a conversion has already been performed in current LI service,
*         Releases memory used by input data descriptor using 
*         <@FUNCTION gcn_FreeDataDesc> 
*       Replaces input data descriptor by temporary data descriptor
*       allocated by <@FUNCTION PRES_pivot2native>
* The pivot format is (HFMT Pivot, CodePage UNICODE).
*
* @RETURNVALUE CSILONG
* Possible returnvalues are: CSISUCCESS (normal) or CSIFAILED (error).
*
* @PARAMETER
* IN    CSICHAR *msgFormat_P
*       The internal syntax of the message (a.k.a XFMT), as previously
*       retrieved in the Directory.
*
* IN    CSICHAR32 msgId
*       The message type identifier.
*
* IN    CSICHAR128 outputHFMT
*       The required final HostFormat of the message.
*
* IN    CSILONG outputCodePage
*       The required final CodePage of the message.
*
* IN    CSILONG *conversion_P
*       If TRUE, the memory used by input data descriptor should be released.
*       Input data descriptor should not be freed if it has never been converted
*       in current LI service. Data descriptor allocated at LI service entrance
*       should be freed through GTX_return.
*
* INOUT CSIPDU **dataDesc_PP
*       The Data Descriptor which is converted.
*
* OUT   CSILONG returnCode_P
*       Result of the operation (same as the function return code)
*
* OUT   CSILONG reasonCode_P
*       Reason why the operation fails (if it fails)
*       If no error occurred, it is LI_ERR_NONE.
*
* @SEEALSO
* <@FUNCTION li_MsgConvertFromPivot>
*/
CSILONG li_MsgConvertFromPivot(
IN      CSICHAR       *msgFormat_P,
IN      CSICHAR32      msgId,
IN      CSICHAR128     outputHFMT,
IN      CSILONG        outputCodePage,
IN      CSILONG       *conversion_P,
INOUT   CSIPDU        **dataDesc_PP,
OUT     CSILONG       *returnCode_P,
OUT     CSILONG       *reasonCode_P
)
{
   /*** LOCAL VARIABLES DECLARATIONS ***/
   /************************************/

   CSILONG retCode        = CSISUCCESS;
   CSILONG reaCode        = CSI_ERR_NONE;
   CSILONG status         = LI_ERR_NONE;
   CSILONG middleCodePage = LI_UNICODE;
   CSILONG response       = FALSE;
   CSIPDU *tmpDataDesc_P  = NULL;

   TRACE_FUNCTION_ENTRY("li_MsgConvertFromPivot");

   /*** PARAMETERS VERIFICATION ***/
   /*******************************/

   if ( (NULL == returnCode_P) || (NULL == reasonCode_P) )
   {
      TRACE_FUNCTION_EXIT("li_MsgConvertFromPivot", CSIFAILED, LI_ERR_INTERNAL);
      return(CSIFAILED);
   }
   *returnCode_P = CSISUCCESS;
   *reasonCode_P = LI_ERR_NONE;

   /*** COMPARE REQUIRED FINAL FORMAT WITH PIVOT ***/
   /************************************************/

   if (LI_ERR_NONE == status)
   { /** BEGIN: IF OK 1 **/

     PRES_fmtcomp(*dataDesc_PP, msgFormat_P, outputHFMT, PRV_PIVOT_HFMT, 
                  middleCodePage, outputCodePage, &response, 
                  returnCode_P, reasonCode_P);
     if (CSIFAILED == *returnCode_P)
     {
       EVR_ERRORLOG(EVR_UNDETERMINATE,
                    (LI_MSG_PRV_ERROR, "PRES_fmtcomp",(*reasonCode_P)));
       status = LI_ERR_PRES_FMTCOMP;
     }
   } /** ENDOF: IF OK 1 **/

   /*** IF OUTPUT HFMT IS NOT PIVOT, CONVERT TO OUTPUT HFMT ***/
   /***********************************************************/

   if ( (LI_ERR_NONE == status) && (TRUE == response) )
   { /** BEGIN: IF OK 2 **/

     PRES_pivot2native(msgFormat_P, msgId, outputHFMT, middleCodePage,
                       outputCodePage, *dataDesc_PP, &tmpDataDesc_P, 
                       returnCode_P, reasonCode_P);
     if (CSIFAILED == *returnCode_P)
     {
       EVR_ERRORLOG(EVR_UNDETERMINATE,
                    (LI_MSG_PRV_ERROR, "PRES_pivot2native",(*reasonCode_P)));
       status = LI_ERR_PRES_PIVOT2NATIVE;
     }

     /** Free dataDesc_PP data descriptor if necessary and **/
     /** replace it by tmpDataDesc_P                       **/
     /*******************************************************/

     if ( (NULL != tmpDataDesc_P) && (NULL != *dataDesc_PP) )
     {
       /* RFA-145 : pass run mode to temporary CSIDD */
       tmpDataDesc_P->Flags |= ((*dataDesc_PP)->Flags & CSI_DD_FLAGS_MODE_SET);

       /* Delete file if no more usefull */
       gcn_DeleteFile(*dataDesc_PP, &retCode, &reaCode);

       /* Release memory used by input data descriptor if a conversion has */
       /* already occured in current LI service */
       if (*conversion_P == TRUE)
       {
          gcn_FreeDataDesc(dataDesc_PP, returnCode_P, reasonCode_P);
          if (CSIFAILED == *returnCode_P)
          {
            EVR_ERRORLOG(EVR_UNDETERMINATE,
                         (LI_MSG_GCN_ERROR, "gcn_FreeDataDesc",(*reasonCode_P)));
            status = LI_ERR_INTERNAL;
          }
       }
       *conversion_P = TRUE;
       *dataDesc_PP = tmpDataDesc_P;
     }
   } /** ENDOF: IF OK 2 **/
 
   /*** THE END ***/
   /***************/

   if (LI_ERR_NONE != status) 
     *returnCode_P = CSIFAILED;
   else 
     *returnCode_P = CSISUCCESS;
   *reasonCode_P = status;

   TRACE_FUNCTION_EXIT("li_MsgConvertFromPivot", *returnCode_P, *reasonCode_P);

   return(*returnCode_P);
}

/********************************************************/
/*** LI: MESSAGE CONVERSION FROM FORMAT A TO FORMAT B ***/
/********************************************************/
/*
* @LOCALFUN li_MsgConvert
*
* @DESCRIPTION Converts message from format A into format B
*
* @ABSTRACT
* Processes data conversion from native format to pivot on originator gateway 
* and from pivot to native format on destination gateway.
* It successively:
*   Determines if current gateway is originator
*   If so, converts data from native format to pivot if needed using 
*   <@LOCALFUN li_MsgConvertToPivot>  
*   Determines if current gateway is destination
*   If so, converts data from pivot to native format if needed using
*   <@LOCALFUN li_MsgConvertFromPivot> 
*
* @RETURNVALUE CSILONG
* Possible returnvalues are: CSISUCCESS (normal) or CSIFAILED (error).
*
* @PARAMETER
* IN    CSICHAR *msgFormat_P
*       The internal syntax of the message (a.k.a XFMT), as previously
*       retrieved in the Directory.
*
* IN    CSICHAR32 msgId
*       The message type identifier.
*
* IN    CSICHAR128 inputHFMT
*       The initial HostFormat of the message.
*
* IN    CSILONG inputCodePage
*       The initial CodePage of the message.
*
* IN    CSICHAR128 outputHFMT
*       The required final HostFormat of the message.
*
* IN    CSILONG outputCodePage
*       The required final CodePage of the message.
*
* INOUT CSIPDU **dataDesc_PP
*       The Data Descriptor which is converted.
*
* OUT   CSILONG returnCode_P
*       Result of the operation (same as the function return code)
*
* OUT   CSILONG reasonCode_P
*       Reason why the operation fails (if it fails)
*       If no error occurred, it is LI_ERR_NONE.
*
*/
CSILONG li_MsgConvert(
IN      LI_MSGCONTEXT *li_MsgContext_P,
IN      CSICHAR       *msgFormat_P,
IN      CSICHAR32      msgId,
IN      CSICHAR128     inputHFMT,
IN      CSILONG        inputCodePage,
IN      CSICHAR128     outputHFMT,
IN      CSILONG        outputCodePage,
INOUT   CSIPDU        **dataDesc_PP,
OUT     CSILONG       *returnCode_P,
OUT     CSILONG       *reasonCode_P
)
{
   /*** LOCAL VARIABLES DECLARATIONS ***/
   /************************************/

   CSILONG processInitial = FALSE; /* it is worth to process initial msg */  
   CSILONG processPivot   = FALSE; /* it is worth to process pivot msg */
   CSILONG conversion     = FALSE;

   TRACE_FUNCTION_ENTRY("li_MsgConvert");

   /*** PARAMETERS VERIFICATION ***/
   /*******************************/

   if ( (NULL == returnCode_P) || (NULL == reasonCode_P) )
   {
      TRACE_FUNCTION_EXIT("li_MsgConvert", CSIFAILED, LI_ERR_INTERNAL);
      return(CSIFAILED);
   }

   *returnCode_P = CSISUCCESS;
   *reasonCode_P = LI_ERR_NONE;

   /*** COMPARE INITIAL FORMAT WITH PIVOT ***/
   /*****************************************/

   /* set the process initial variable */
   if ( li_MsgContext_P->Direction == LI_REQUEST)  
      processInitial = ( ! ( 
	(li_MsgContext_P->TraceInfo_P->NbOfPrevGateways == 2L) &&
        (li_MsgContext_P->EventType == CSI_LOCAL_EVENT) ) );
   else  /* Direction is LI_REPLY */
      processInitial = (li_MsgContext_P->EventType == CSI_LOCAL_EVENT); 

   if (processInitial)
   {
     /* It is the first LI  we are going through */
     li_MsgConvertToPivot(msgFormat_P, msgId, inputHFMT, inputCodePage,
                          &conversion, dataDesc_PP,
                          returnCode_P, reasonCode_P);
   } /* end if processInitial */

   /*** COMPARE REQUIRED FINAL FORMAT WITH PIVOT ***/
   /************************************************/

   if (CSISUCCESS == *returnCode_P)
   {
     /* set the processPivot variable */
     if (li_MsgContext_P->Direction == LI_REQUEST) 
        processPivot = (li_MsgContext_P->EventType == CSI_LOCAL_EVENT);
     else /* ...Direction is LI_REPLY */ 
        processPivot = (   (   (li_MsgContext_P->EventType == CSI_REMOTE_EVENT)
                            || (li_MsgContext_P->EventType == CSI_DIRECT_REMOTE_EVENT)
                           )
                        || (   (li_MsgContext_P->EventType == CSI_LOCAL_EVENT)
                            && (li_MsgContext_P->TraceInfo_P->NbOfPrevGateways == 1L)
                           )
                       );

     if (processPivot) 
     {
       /* We are on the destination gateway, so  we have to perform conversion 
          to output format */
       /* If a VAS has been previously performed on destination gateway, */
       /* message format could be different from pivot. We must first */
       /* convert message to pivot if needed */
 
       if (!(processInitial))
       {
         /* If this LI is also the first LI we go through, the data have been
            previously encoded in pivot format, no need to call again the
            conversion function. */

         li_MsgConvertToPivot(msgFormat_P, msgId, inputHFMT, inputCodePage,
                              &conversion, dataDesc_PP,
                              returnCode_P, reasonCode_P);
       }
       if (CSISUCCESS == *returnCode_P)
       {

         li_MsgConvertFromPivot(msgFormat_P, msgId, outputHFMT, outputCodePage,
                                &conversion, dataDesc_PP, 
                                returnCode_P, reasonCode_P);
       }
     } /* end if  */
   }

   /*** THE END ***/
   /***************/

   TRACE_FUNCTION_EXIT("li_MsgConvert", *returnCode_P, *reasonCode_P);

   return(*returnCode_P);
}

/*****************************************************************************/
